#include <ggi/internal/gpf.h>
#include <ggi/internal/gpf_pipe.h>
#include <ggi/internal/gpf_internal.h>
#include <ggi/internal/gpf_debug.h>
#include <ggi/internal/gpf_helpers.h>
#include <ggi/internal/gpf_datamgr.h>

#include <stdlib.h>
#include <string.h>

size_t gpfConsumeData(struct gpf_handle_pipeline_t* hnd_pipeline,
		      struct gpf_pipeline_item_t *pipe, 
		      size_t wrote_bytes,
		      enum gpf_pipeline_item_type_t type)
{
	size_t ret = 0;
  
	switch (type) {
	case GPF_PIPE_INPUT :
		/* input target */
		ret = pipe->writebytes = wrote_bytes;

		GPFDPRINT_TRANSFER("gpfConsumeData (input): %i bytes finally written, %i bytes to cache\n",
			pipe->writebytes, pipe->readbytes - pipe->writebytes);
    
		if (pipe->readbytes > pipe->writebytes) {
			PIPE_READCACHEDATA(pipe, pipe->readbytes - pipe->writebytes);
		}	/* if */
		PIPE_READBUFPOS(pipe) = 0;
    
		hnd_pipeline->pipe_index = hnd_pipeline->pipe_index->next;
		break;
    
	case GPF_PIPE_OUTPUT:
		/* output target */
		ret = pipe->writebytes;

		GPFDPRINT_TRANSFER("gpfConsumeData (output): %i bytes transfered, %i bytes must be cached\n",
			pipe->writebytes, PIPE_WRITEBUFPOS(pipe->prev) - pipe->writebytes);
    
		if (PIPE_WRITEBUFPOS(pipe->prev) < pipe->writebytes) {
			PIPE_READCACHEDATA(pipe, pipe->writebytes -
					PIPE_WRITEBUFPOS(pipe->prev) +
					PIPE_READCACHESIZE(pipe));
		} else {
			PIPE_READCACHEDATA(pipe,
					PIPE_WRITEBUFPOS(pipe->prev) -
					pipe->writebytes);
		}	/* if */
    
		if (PIPE_WRITEBUFPOS(pipe->prev) <= pipe->writebytes) {
			hnd_pipeline->pipe_index = hnd_pipeline->pipe_index->prev;
			pipe->readbytes = pipe->writebytes = 0;
		}	/* if */
    
		GPFDPRINT_TRANSFER("gpfConsumeData (output): %i bytes cached\n",
			PIPE_READCACHESIZE(pipe));
		break;
	}	/* switch */

	return ret;
}	/* gpfConsumeData */



size_t gpfProduceData(struct gpf_handle_pipeline_t* hnd_pipeline,
		      struct gpf_pipeline_item_t *pipe, 
		      size_t frame_size,
		      struct timeval* transfered_timestamps,
		      enum gpf_pipeline_item_type_t type)
{
	size_t transfer_size = 0;

	switch (type) {
	case GPF_PIPE_INPUT:

		/* input target */
		LIB_ASSERT(pipe->buf_conv == NULL);
		LIB_ASSERT(pipe->buf_conv_size == 0);
		pipe->buf_conv = PIPE_READBUF(pipe);
		pipe->buf_conv_size = PIPE_READBUFSIZE(pipe);
    
		/* convert timeval to bytes to transfer */
		transfer_size = _gpf_gettransfersize(frame_size,
						transfered_timestamps);
		LIMIT_TRANSFERSIZE;
		break;

	case GPF_PIPE_OUTPUT:
		/* output target */

		/* convert timeval to bytes to transfer */
		transfer_size = _gpf_gettransfersize(frame_size,
						transfered_timestamps);   
		GET_TARGET_TRANSFERSIZE;
		LIMIT_TRANSFERSIZE;
		REALLOC_BUFCONVSIZE(transfer_size);
    
		GPFDPRINT_TRANSFER("gpfProduceData (output): try to transfer %i bytes, %i bytes are cached\n",
			transfer_size, PIPE_READCACHESIZE(pipe));
		GPFDPRINT_TRANSFER("gpfProduceData (output): %i bytes in read buffer (minus cached bytes)\n",
			PIPE_WRITEBUFPOS(pipe->prev));
    
		FIXME("%s:%s:%i: When interframe delay not exceeded,"
			" then give control back to application here\n",
			DEBUG_INFO);

		PIPE_READBUFPOS(pipe) = 0;
		break;
	}	/* switch */
  
	return transfer_size;
}	/* gpfProduceData */
